#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : ieee_ra.py
@Time    : 2022/4/6 21:10
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2022 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import csv
import re
from pathlib import Path
from typing import Dict, List

from django.db import models


class OuiRegistry(models.TextChoices):
    MA_L = 'MA-L'
    MA_M = 'MA-M'
    MA_S = 'MA-S'
    IAB = 'IAB'
    CID = 'CID'


class OuiParser:
    headers = [
        'registry', 'assignment', 'organization_name',
        'organization_address_1', 'organization_address_2', 'organization_address_3'
    ]
    default_files: Dict[OuiRegistry, list] = {
        OuiRegistry.MA_L: 'oui.txt',
        OuiRegistry.MA_M: 'mam.txt',
        OuiRegistry.MA_S: 'oui36.txt',
        OuiRegistry.IAB: 'iab.txt',
        OuiRegistry.CID: 'cid.txt',
    }

    default_registries = [
        OuiRegistry.MA_L, OuiRegistry.MA_M, OuiRegistry.MA_S, OuiRegistry.IAB, OuiRegistry.CID
    ]

    def txt_content_parser(self,
                           content: str, /, *,
                           registry: OuiRegistry = None,
                           deduplicate: bool = True
                           ) -> list:
        _rows = []
        _dict = {}
        try:
            for mg in content.split('\n\n')[1:]:
                splitlines = mg.splitlines()
                # hex_str = re.search(r'^[A-Z0-9]{2}-[A-Z0-9]{2}-[A-Z0-9]{2}', splitlines[0].strip()).group().strip()
                [hex_str, organization] = [_x.strip() for _x in splitlines[0].split('(hex)')]
                base16_str = re.search(r'^[A-Z0-9-]+', splitlines[1].strip()).group().strip()
                if registry in [OuiRegistry.MA_L, OuiRegistry.CID]:
                    assignment = base16_str
                elif registry == OuiRegistry.MA_M:
                    assignment = hex_str.replace('-', '') + base16_str[0]
                elif registry in [OuiRegistry.MA_S, OuiRegistry.IAB]:
                    assignment = hex_str.replace('-', '') + base16_str[0:3]
                else:
                    assignment = ''
                # organization = re.search(r'\t.+$', splitlines[0].strip()).group().strip()
                address_new: list = [a.strip() for a in splitlines[2:]]
                item = [registry.value, assignment, organization] + address_new
                if deduplicate:
                    _dict[f'{registry.name}-{assignment}'] = item
                else:
                    _rows.append(item)
            if deduplicate:
                _rows = list(_dict.values())
            _rows.sort()
        except Exception as e:
            pass
        finally:
            return _rows

    def load_txt_files(self,
                       folder: Path = None,
                       registries: List[OuiRegistry] = None,
                       files: Dict[OuiRegistry, list] = None
                       ) -> Dict[OuiRegistry, list]:
        _registries = self.default_registries if not registries else registries
        _files = self.default_files if not files else self.default_files.copy().update(files)
        _dict = {}
        for registry in _registries:
            oui_path = folder / _files.get(registry, '')
            with oui_path.open(mode='r', encoding='utf-8') as f:
                content = f.read()
                _dict[registry] = self.txt_content_parser(content, registry=registry)
        return _dict

    def save_csv(self, path: Path, /, data: Dict[OuiRegistry, list]):
        with open(str(path), 'w', encoding='utf-8', newline='') as f:
            f_csv = csv.writer(f)
            f_csv.writerow(self.headers)
            for l in data.values():
                f_csv.writerows(l)

    def load_csv(self, path: Path, /):
        with open(str(path), encoding='utf-8') as f:
            # rows = csv.DictReader(f)
            # return list(rows)
            row = csv.reader(f)
            headers = next(row)
            pass

    def compare(self, latest: Path, older: Path):
        latest_dict = {}
        older_dict = {}
        items_new = []  # new
        items_deleted = []  # deleted
        items_same = []  # same
        items_updated = []  # update
        with open(str(latest), encoding='utf-8') as f:
            row = csv.reader(f)
            headers = next(row)
            for _r in row:
                latest_dict[f'{_r[0]}-{_r[1]}'] = _r
            pass
        with open(str(older), encoding='utf-8') as f:
            row = csv.reader(f)
            headers = next(row)
            for _r in row:
                older_dict[f'{_r[0]}-{_r[1]}'] = _r
            pass
        for _latest_k, _latest_v in latest_dict.items():
            if _latest_k in older_dict:  # 若新列表中存在于旧列表
                if _latest_v == older_dict.get(_latest_k):  # 如果值相同
                    items_same.append(_latest_v)
                else:  # 如果值不同
                    items_updated.append(_latest_v)
                older_dict.pop(_latest_k)
            else:  # 若新列表键名不存在于旧列表
                items_new.append(_latest_v)
        items_deleted.extend(older_dict.values())
        return items_new, items_updated, items_same, items_deleted


def oui_list_compare(latest, older: list = None):
    pass


if __name__ == '__main__':
    # MA-L 有重复的项目 如：0001C8、080030。需要特别注意。
    data_dir: Path = Path(__file__).parent.parent / 'tests' / 'parsers' / 'data'
    ieee_dir: Path = data_dir / 'standards-oui.ieee.org'
    sl = list(ieee_dir.iterdir())
    sl.sort(reverse=True)
    last_dir: Path = sl[0]
    older_dir: Path = sl[1]
    parser = OuiParser()
    # rows_latest = parser.load_txt_files(folder=last_dir, registries=[OuiRegistry.MA_L])
    # r1 = parser.save_csv(last_dir / 'ieee_ra.csv', rows_latest)
    # rows_older = parser.load_txt_files(folder=older_dir, registries=[OuiRegistry.MA_L])
    # r2 = parser.save_csv(older_dir / 'ieee_ra.csv', rows_older)

    # latest_rows = parser.load_csv(last_dir / 'ieee_ra.csv')

    items_new, items_updated, items_same, items_deleted = parser.compare(last_dir / 'ieee_ra.csv',
                                                                         older_dir / 'ieee_ra.csv')
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : ses.py
@Time    : 2021/8/25 14:50
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2021 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import os
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import List

import boto3
from botocore.exceptions import ClientError


class Attachment:
    _file_name: str
    _att: MIMEApplication


class SesMail:

    def __init__(self,
                 recipient: str = None,
                 subject: str = None,
                 body_text: str = None,
                 body_html: str = None,
                 charset: str = None,
                 attachments: List[Attachment] = None,
                 sender: str = None,
                 ses_region: str = None,
                 ses_user: str = None,
                 ses_secret: str = None
                 ):
        self._recipient: str = recipient if recipient else ''
        self._subject: str = subject if subject else ''
        self._body_text: str = body_text if body_text else ''
        self._body_html: str = body_html if body_html else ''
        self._charset: str = charset if charset else 'utf-8'
        self._attachments: List[Attachment] = attachments if attachments else []
        self._sender: str = sender if sender else os.getenv('SES_SENDER', 'Amazon <notice@cy-mail.itool.co>')
        self._init_client(ses_region, ses_user, ses_secret)
        self._init_msg()

    def _init_client(self,
                     ses_region: str = None,
                     ses_user: str = None,
                     ses_secret: str = None,
                     ):
        _SES_REGION: str = ses_region if ses_region else os.getenv('SES_REGION', 'us-west-2')
        _SES_USER: str = ses_user if ses_user else os.getenv('SES_USER', '')
        _SES_SECRET: str = ses_secret if ses_secret else os.getenv('SES_SECRET', '')
        self._client: boto3.client = boto3.client(
            service_name='ses',
            region_name=_SES_REGION,
            aws_access_key_id=_SES_USER,
            aws_secret_access_key=_SES_SECRET
        )

    def _init_msg(self):
        self._msg: MIMEMultipart = MIMEMultipart('mixed')
        self._msg['Subject'] = self._subject
        self._msg['From'] = self._sender
        self._msg['To'] = self._recipient
        msg_body = MIMEMultipart('alternative')
        text_part = MIMEText(self._body_text.encode(self._charset), 'plain', self._charset)
        html_part = MIMEText(self._body_html.encode(self._charset), 'html', self._charset)
        msg_body.attach(text_part)
        msg_body.attach(html_part)
        self._msg.attach(msg_body)
        for attachment in self._attachments:
            att = attachment._att
            file_name = attachment._file_name
            att.add_header('Content-Disposition', 'attachment', filename=file_name)
            self._msg.attach(att)

    def send(self):
        try:
            response = self._client.send_raw_email(
                Source=self._sender,
                Destinations=[self._recipient],
                RawMessage={
                    'Data': self._msg.as_string(),
                },
            )
        except ClientError as e:
            print(e.response['Error']['Message'])
        else:
            print('Email sent! Message ID:'),
            return response['ResponseMetadata']['RequestId']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : others.py
@Time    : 2022/3/16 11:51
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2022 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import threading
from functools import wraps


class Singleton(object):
    """
    单例模式（非线程安全）
    """
    _instance: object

    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls)
        return cls._instance


def singleton_simple(cls):
    """
    单例模式装饰器（非线程安全）
    :param cls:
    :return:
    """
    _instance = {}

    @wraps(cls)
    def _singleton(*args, **kwargs):
        if cls not in _instance:
            _instance[cls] = cls(*args, **kwargs)
        return _instance[cls]

    return _singleton


def singleton(threadsafe=False):
    """
    单例模式装饰器（可选线程安全）
    :param cls:
    :return:
    """
    _instance = {}
    thread_lock = threading.Lock()

    def singleton_decorator(cls):
        @wraps(cls)
        def _singleton(*args, **kwargs):
            if threadsafe:
                with thread_lock:
                    if cls not in _instance:
                        _instance[cls] = cls(*args, **kwargs)
                    return _instance[cls]
            else:
                if cls not in _instance:
                    _instance[cls] = cls(*args, **kwargs)
                return _instance[cls]

        return _singleton

    return singleton_decorator


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : ty.py
@Time    : 2021/3/28 16:33
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2021 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import datetime
import json
import os
import random
import time
from urllib.request import urlopen

import requests

from we_pyutils.t.ip import get_my_ip

DEFAULT_GET_IP_URL = os.getenv('PROXY_TY_GET_IP_URL', '')
DEFAULT_GET_PACK_INFO_URL = os.getenv('PROXY_TY_GET_PACK_INFO_URL', '')
DEFAULT_ADD_WHITE_IP_URL = os.getenv('PROXY_TY_ADD_WHITE_IP_URL', '')
DEFAULT_DEL_WHITE_IP_URL = os.getenv('PROXY_TY_DEL_WHITE_IP_URL', '')
DEFAULT_ACCOUNT_NAME = os.getenv('PROXY_TY_ACCOUNT_NAME', '')
DEFAULT_ACCOUNT_KEY = os.getenv('PROXY_TY_ACCOUNT_KEY', '')
DEFAULT_NEEK = os.getenv('PROXY_TY_NEEK', '')
DEFAULT_APPKEY = os.getenv('PROXY_TY_APPKEY', '')

ADD_WHITE_IP_URL = f'ty-http-d.hamir.net/index/white/add?neek={"ACCOUNT_NAME"}&appkey={"ACCOUNT_KEY"}&white={"IP"}'
DEL_WHITE_IP_URL = f'ty-http-d.hamir.net/index/white/del?neek={"ACCOUNT_NAME"}&appkey={"ACCOUNT_KEY"}&white={"IP"}'
LIST_WHITE_IP_URL = f'ty-http-d.hamir.net/index/index/white_list?neek={"NEEK"}&appkey={"APPKEY"}'
GET_PACK_INFO_URL = f'ty-http-d.hamir.net/index/index/get_my_pack_info?neek={"NEEK"}&appkey={"APPKEY"}'
GET_PACKAGE_BALANCE_URL = f'ty-http-d.hamir.net/index/index/get_my_package_balance?neek={"NEEK"}&appkey={"APPKEY"}&ac={"AC"}'

TY_DOMAIN: str = os.getenv('PROXY_TY_DOMAIN', 'ty-http-d.hamir.net')
TY_TIQU_DOMAIN: str = os.getenv('PROXY_TY_TIQU_DOMAIN', 'http.tiqu.alibabaapi.com')

TY_ACCOUNT_NAME: str = os.getenv('PROXY_TY_ACCOUNT_NAME', '')
TY_ACCOUNT_KEY: str = os.getenv('PROXY_TY_ACCOUNT_KEY', '')
TY_NEEK: str = os.getenv('PROXY_TY_NEEK', '')
TY_APPKEY: str = os.getenv('PROXY_TY_APPKEY', '')


class ProxyIP:
    """
    自定义太阳代理IP格式
    """
    ip: str
    port: int
    expire_time: datetime
    city: str
    isp: str
    used_times: int
    ip_dict: dict

    def __init__(self,
                 ip: str = None,
                 port: int = None,
                 expire_time=None,
                 city: str = None,
                 isp: str = None,
                 used_times: int = 0,
                 ip_dict: dict = None
                 ):
        if ip_dict:
            if 'ip' in ip_dict and ip_dict['ip']:
                self.ip = ip_dict['ip']
            if 'port' in ip_dict and ip_dict['port']:
                self.port = int(ip_dict['port'])
            if 'expire_time' in ip_dict and ip_dict['expire_time']:
                self.expire_time = self.format_expire_time(ip_dict['expire_time'])
            if 'city' in ip_dict and ip_dict['city']:
                self.city = ip_dict['city']
            if 'isp' in ip_dict and ip_dict['isp']:
                self.isp = ip_dict['isp']
            if 'used_times' in ip_dict and ip_dict['used_times']:
                self.used_times = ip_dict['used_times']
            else:
                self.used_times = used_times
        else:
            if ip:
                self.ip = ip
            else:
                raise ValueError
            if port:
                self.port = port if isinstance(port, int) else int(port)
            else:
                self.port = port
            if isinstance(expire_time, str):
                self.expire_time = self.format_expire_time(expire_time)
            else:
                self.expire_time = expire_time
            if used_times:
                self.used_times = used_times if isinstance(used_times, int) else int(used_times)
            else:
                self.used_times = used_times
            self.city = city
            self.isp = isp

    def __str__(self):
        return f'IP:{self.ip}:{self.port}, Used:{self.used_times} times.'

    def __eq__(self, obj):
        return self.__dict__ == obj.__dict__

    def use_ip_str(self):
        self.used_times_add()
        return self.proxy_str()

    def proxy_str(self):
        return f'{self.ip}:{self.port}'

    def get_used_times(self):
        return self.used_times

    def used_times_add(self, times=1):
        self.used_times += times

    @staticmethod
    def format_expire_time(expire_time_str: str) -> datetime:
        return datetime.datetime.strptime(expire_time_str, '%Y-%m-%d %H:%M:%S')


class ProxyIPPool:
    def __init__(self, ip_dict: dict = None):
        self.ip_pool: dict = {} if ip_dict is None else ip_dict

    def __len__(self):
        return len(self.ip_pool)

    def __eq__(self, obj):
        return self.ip_pool == obj.ip_pool

    def __dict__(self):
        return self.ip_pool

    def enlarge_ip_pool(self, ip_dict: dict):
        self.ip_pool.update(ip_dict)
        return self.ip_pool

    def enlarge_ip_pool_b(self, ip_list: list):
        ip_pool_tmp = {}
        for ip_item in ip_list:
            ip = ProxyIP(ip_dict=ip_item)
            ipstr = ip.ip
            ip_pool_tmp[ip.ip] = ip
        self.ip_pool.update(ip_pool_tmp)
        return self.ip_pool

    def add_ip(self, ip: ProxyIP):
        self.ip_pool[ip.ip] = ip
        return ip

    def update_ip(self, ip: ProxyIP):
        self.ip_pool[ip.ip] = ip
        return ip

    def remove_ip(self, ip: str):
        del self.ip_pool[ip]

    def random_choice_ip(self, count=1) -> ProxyIP:
        p = list(self.ip_pool.values())
        return random.choice(p)
        # return random.sample(p, count) # 选择多个IP（返回list）

    def get_ip_pool(self):
        return self.ip_pool


class Singleton(object):
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls)
        return cls._instance


class ProxyTYClient(Singleton):
    get_pack_info_count = 0
    max_ip_used_times = 65
    packs_info = {}

    def __init__(self, get_ip_url=None, get_pack_info_url=None,
                 add_white_ip_url=None, del_white_ip_url=None,
                 account_name=None, account_key=None, neek=None, appkey=None
                 ):
        self.get_ip_url = get_ip_url if get_ip_url else DEFAULT_GET_IP_URL
        self.get_pack_info_url = get_pack_info_url if get_pack_info_url else DEFAULT_GET_PACK_INFO_URL
        self.add_white_ip_url = add_white_ip_url if add_white_ip_url else DEFAULT_ADD_WHITE_IP_URL
        self.del_white_ip_url = del_white_ip_url if del_white_ip_url else DEFAULT_DEL_WHITE_IP_URL
        self.account_name = account_name if account_name else DEFAULT_ACCOUNT_NAME
        self._account_key = account_key if account_key else DEFAULT_ACCOUNT_KEY
        self.neek = neek if neek else DEFAULT_NEEK
        self._appkey = appkey if appkey else DEFAULT_APPKEY

    def get_ips(self, pack_id, ip_nums=1, proxy_regions=None, times=0):
        if times >= 10:
            return False
        if proxy_regions is None:
            proxy_regions = []
        print(f'Get new proxy IPs ... ({times} times)')
        try:
            url = f'{self.get_ip_url}' + \
                  f'?num={ip_nums}&type=2&pro=0&city=0&yys=0&port=11&' + \
                  f'pack={pack_id}&ts=1&ys=1&cs=1&lb=1&sb=0&pb=4&mr=0&regions=' + \
                  ','.join(proxy_regions)
            res = urlopen(url, timeout=2).read().decode('utf-8')
            res_json = json.loads(res)

            if res_json.get('code', -1) > 0:
                my_ip = get_my_ip()
                if my_ip:
                    self.add_white_ip(my_ip)
                time.sleep(5)
                return self.get_ips(pack_id, ip_nums, proxy_regions, times=times + 1)
            elif res_json.get('success', False):
                ips = res_json.get('data', {})
                return ips
            return False
        except Exception as e:
            print(f'Get new proxy IPs FAILED. {e}')
            return False

    def get_pack_info(self, count=0):
        try:
            if len(self.packs_info) > 0:
                return self.packs_info
            if count >= 10:
                return False
            self.get_pack_info_count += 1
            print(f'Get Proxy Pack Info ({count} times)')
            gpi_url = f'{self.get_pack_info_url}?neek={self.neek}&appkey={self._appkey}'
            gpi_json = urlopen(gpi_url, timeout=4).read().decode('utf-8')
            self.packs_info = json.loads(gpi_json)
            return self.packs_info
        except Exception as e:
            print(f'No Available Packs!!! 10s Later Retry...({count} times)')
            print(e)
            my_ip = get_my_ip()
            if my_ip:
                self.add_white_ip(my_ip)
            time.sleep(10)
            return self.get_pack_info(count=count + 1)

    def check_and_add_white_ip(self):
        my_ip = get_my_ip()
        if my_ip:
            self.add_white_ip(my_ip)
        time.sleep(5)
        return my_ip

    def add_white_ip(self, ip):
        try:
            add_ip_url = f'{self.add_white_ip_url}?neek={self.account_name}&appkey={self._account_key}&white={ip}'
            res = urlopen(add_ip_url, timeout=2).read().decode('utf-8')
            res_json = json.loads(res)
            if res_json['success']:
                print(f'Add white ip:{ip}')
                return True
        except Exception as e:
            print(f'Add white ip ({ip}) FAILED.')
            print(e)
            return False

    def del_white_ip(self):
        pass

    @staticmethod
    def check_pack_usable(pack):
        all_num = int(pack['all_num'])
        balance = int(pack['balance'])
        name = pack['name']
        near_fill_time = pack['near_fill_time']
        now_timestamps = time.time()
        if name == '包年套餐':
            if all_num > 0:
                return True
            return False
        elif name == '单月包量套餐':
            if (balance > 0) and (near_fill_time is None or now_timestamps < near_fill_time):
                return True
        else:
            return False

    def check_ip_usable(self, ip: ProxyIP, expire_seconds: int = 120):
        if ip.used_times >= self.max_ip_used_times:
            return False
        now = datetime.datetime.now()
        if ip.expire_time <= now:
            return False
        c = ip.expire_time - now
        if c.seconds < expire_seconds:
            return False
        return True


class ProxyTYMixin():
    domain: str
    domain_tiqu: str

    neek: str
    account_name: str
    appkey: str
    account_key: str
    packs_info: dict = {}

    def _get_settings(self):
        pass

    def _request(self, url: str, /, *, method: str = 'get', params: dict = None, data: dict = None):
        if method == 'get':
            res = requests.get(url, params=(params or {}))
            return res.json()
        elif method == 'post':
            res = requests.post(url, params=(params or {}), data=(data or {}))
            return res.json()
        return None

    def list_white_ip(self) -> dict:
        url = f'https://{self.domain}/index/index/white_list'
        params = {
            'neek': self.neek,
            'appkey': self.appkey,
        }
        try:
            res_json = self._request(url, params=params)
            if res_json.get('msg') == 'ok':
                res_json_data = res_json.get('data') or {'lists': []}
                return res_json_data
        except Exception as e:
            print(f'List white ip FAILED.')
        return {}

    def add_white_ip(self, ip: list) -> bool:
        join_ip = ','.join(ip)
        url = f'https://{self.domain}/index/white/add'
        params = {
            'neek': self.account_name,
            'appkey': self.account_key,
            'white': join_ip
        }
        try:
            res_json = self._request(url, params=params)
            if res_json.get('success'):
                print(f'Add white ip:{join_ip}')
                return True
        except Exception as e:
            print(f'Add white ip ({join_ip}) FAILED.')
            print(e)
            return False
        return False

    def get_pack_info(self, count=0):
        url = f'https://{self.domain}/index/index/get_my_pack_info'
        params = {
            'neek': self.neek,
            'appkey': self.appkey,
        }
        try:
            if len(self.packs_info) > 0:
                return self.packs_info
            if count >= 10:
                return None
            # self.get_pack_info_count += 1
            print(f'Get Proxy Pack Info ({count} times)')
            res_json = self._request(url, params=params)
            return res_json
        except Exception as e:
            print(f'No Available Packs!!! 10s Later Retry...({count} times)')
            print(e)
            my_ip = get_my_ip()
            if my_ip:
                self.add_white_ip(my_ip)
            time.sleep(10)
            return self.get_pack_info(count=count + 1)

    def get_ips(self, pack_id: int, ip_nums: int = 1, proxy_regions: list = None, times: int = 0):
        if times >= 10:
            return False
        if proxy_regions is None:
            proxy_regions = []
        proxy_regions_join = ','.join(proxy_regions)
        # 参数文档：https://www.tyhttp.com/help/218.html
        url = f'http://{self.domain_tiqu}/getip'
        params = {
            'num': ip_nums,  # 提取的IP数量
            'type': 2,  # 数据格式(1:TXT 2:JSON 3:html)
            'pack': pack_id,  # 用户套餐ID
            'port': 1,  # 代理协议(1表示HTTP 11表示HTTPS 2表示SOCK5)
            'ts': 1,  # 是否显示IP过期时间(1显示 2不显示)
            'ys': 1,  # 是否显示IP运营商(1显示 2不显示)
            'cs': 1,  # 是否显示位置(1显示 2不显示)
            'lb': 1,  # 分隔符(1:\r\n 2:/br 3:\r 4:\n 5:\t 6 :自定义)
            'pb': 4,  #
            'regions': proxy_regions_join,  #
            # 'pro': 420000,  # 代表省份
            # 'city': 420000,  # 代表城市
        }
        try:
            print(f'Get new proxy IPs ... ({times} times)')
            res_json = self._request(url, params=params)
            if res_json.get('code', -1) > 0:
                my_ip = get_my_ip()
                if my_ip:
                    self.add_white_ip(my_ip)
                time.sleep(5)
                return self.get_ips(pack_id, ip_nums, proxy_regions, times=times + 1)
            elif res_json.get('success', False):
                ips = res_json.get('data', [])
                return ips
            return False
        except Exception as e:
            print(f'Get new proxy IPs FAILED. {e}')
            return False


if __name__ == '__main__':
    ty = ProxyTYMixin()
    # list_white_ip = ty.list_white_ip()
    # add_white_ip = ty.add_white_ip(['111.197.248.175'])
    # get_pack_info = ty.get_pack_info()
    get_ips = ty.get_ips(51965, 1)
    pass
    # 功能测试
    import environ
    from we_pyutils.env import GetEnv

    BASE_DIR = os.path.dirname(os.path.abspath(__file__))
    PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.dirname(BASE_DIR)))
    HOME_DIR = os.path.expanduser('~')
    dir_list = [
        BASE_DIR,
        PROJECT_DIR,
        HOME_DIR,
    ]

    e = GetEnv(dir_list)
    env_path = e.get_env()
    env = environ.Env()
    if env_path:
        env.read_env(env_path)
    get_ip_url = env.str('PROXY_TY_GET_IP_URL', '')
    get_pack_info_url = env.str('PROXY_TY_GET_PACK_INFO_URL', '')
    add_white_ip_url = env.str('PROXY_TY_ADD_WHITE_IP_URL', '')
    del_white_ip_url = env.str('PROXY_TY_DEL_WHITE_IP_URL', '')
    account_name = env.str('PROXY_TY_ACCOUNT_NAME', '')
    account_key = env.str('PROXY_TY_ACCOUNT_KEY', '')
    neek = env.str('PROXY_TY_NEEK', '')
    appkey = env.str('PROXY_TY_APPKEY', '')

    proxy = ProxyTYClient(
        get_ip_url, get_pack_info_url,
        add_white_ip_url, del_white_ip_url,
        account_name, account_key, neek, appkey
    )
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : amazon.py
@Time    : 2020-02-09 8:53
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import re
from decimal import Decimal, getcontext


class Singleton(object):
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls)
        return cls._instance


# 参考：http://docs.developer.amazonservices.com/en_US/dev_guide/DG_Endpoints.html
MARKETPLACES = {
    # North America
    'BR': 'www.amazon.com.br',  # Brazil
    'CA': 'www.amazon.ca',  # Canada
    'MX': 'www.amazon.com.mx',  # Mexico
    'US': 'www.amazon.com',  # US
    # Europe region
    'AE': 'www.amazon.ae',  # United Arab Emirates (U.A.E.)
    'DE': 'www.amazon.de',  # Germany
    'EG': '',  # Egypt
    'ES': 'www.amazon.es',  # Spain
    'FR': 'www.amazon.fr',  # France
    'UK': 'www.amazon.co.uk',  # UK
    'GB': 'www.amazon.co.uk',  # UK
    'IN': 'www.amazon.in',  # India
    'IT': 'www.amazon.it',  # Italy
    'NL': 'www.amazon.nl',  # Netherlands
    'PL': 'www.amazon.pl',  # Poland
    'SA': 'www.amazon.sa',  # Saudi Arabia
    'SE': 'www.amazon.se',  # Sweden
    'TR': 'www.amazon.com.tr',  # Turkey
    # Far East region
    'SG': 'www.amazon.sg',  # Singapore
    'AU': 'www.amazon.com.au',  # Australia
    'JP': 'www.amazon.co.jp',  # Japan
    # China
    'CN': 'www.amazon.cn',  # China
}

MARKETPLACE_IDS = {
    # North America
    'BR': 'A2Q3Y263D00KWC',  # Brazil
    'CA': 'A2EUQ1WTGCTBG2',  # Canada
    'MX': 'A1AM78C64UM0Y8',  # Mexico
    'US': 'ATVPDKIKX0DER',  # US
    # Europe region
    'AE': 'A2VIGQ35RCS4UG',  # United Arab Emirates (U.A.E.)
    'DE': 'A1PA6795UKMFR9',  # Germany
    'EG': 'ARBP9OOSHTCHU',  # Egypt
    'ES': 'A1RKKUPIHCS9HS',  # Spain
    'FR': 'A13V1IB3VIYZZH',  # France
    'UK': 'A1F83G8C2ARO7P',  # UK
    'GB': 'A1F83G8C2ARO7P',  # UK
    'IN': 'A21TJRUUN4KGV',  # India
    'IT': 'APJ6JRA9NG5V4',  # Italy
    'NL': 'A1805IZSGTT6HS',  # Netherlands
    'PL': 'A1C3SOZRARQ6R3',  # Poland
    'SA': 'A17E79C6D8DWNP',  # Saudi Arabia
    'SE': 'A2NODRKZP88ZB9',  # Sweden
    'TR': 'A33AVAJ2PDY3EV',  # Turkey
    # Far East region
    'SG': 'A19VAU5U5O7RUS',  # Singapore
    'AU': 'A39IBJ37TRP1C6',  # Australia
    'JP': 'A1VC38T7YXB528',  # Japan
    # China
    'CN': 'AAHKV2X7AFYLW',
}

MWS_ENDPOINTS = {
    # North America
    'BR': 'https://mws.amazonservices.com',  # Brazil
    'CA': 'https://mws.amazonservices.ca',  # Canada
    'MX': 'https://mws.amazonservices.com.mx',  # Mexico
    'US': 'https://mws.amazonservices.com',  # US
    # Europe region
    'AE': 'https://mws.amazonservices.ae',  # United Arab Emirates (U.A.E.)
    'DE': 'https://mws-eu.amazonservices.com',  # Germany
    'EG': 'https://mws-eu.amazonservices.com',  # Egypt
    'ES': 'https://mws-eu.amazonservices.com',  # Spain
    'FR': 'https://mws-eu.amazonservices.com',  # France
    'UK': 'https://mws-eu.amazonservices.com',  # UK
    'GB': 'https://mws-eu.amazonservices.com',  # UK
    'IN': 'https://mws.amazonservices.in',  # India
    'IT': 'https://mws-eu.amazonservices.com',  # Italy
    'NL': 'https://mws-eu.amazonservices.com',  # Netherlands
    'PL': 'https://mws-eu.amazonservices.com',  # Poland
    'SA': 'https://mws-eu.amazonservices.com',  # Saudi Arabia
    'SE': 'https://mws-eu.amazonservices.com',  # Sweden
    'TR': 'https://mws-eu.amazonservices.com',  # Turkey
    # Far East region
    'SG': 'https://mws-fe.amazonservices.com',  # Singapore
    'AU': 'https://mws.amazonservices.com.au',  # Australia
    'JP': 'https://mws.amazonservices.jp',  # Japan
    # China
    'CN': 'https://mws.amazonservices.com.cn',
}


class MarketPlaces(Singleton):
    # North America
    BR = None
    CA = None
    MX = None
    US = None
    # Europe region
    AE = None
    DE = None
    EG = None
    ES = None
    FR = None
    UK = None  # GB
    GB = None
    IN = None
    IT = None
    NL = None
    PL = None
    SA = None
    SE = None
    TR = None
    # Far East region
    SG = None
    AU = None
    JP = None
    # China
    CN = None

    def __init__(self):
        for country_code, domain in MARKETPLACES.items():
            setattr(self, country_code, domain)

    @staticmethod
    def all():
        return MARKETPLACES

    @staticmethod
    def domains():
        return list(MARKETPLACES.values())

    @staticmethod
    def ids():
        return list(MARKETPLACE_IDS.values())

    @staticmethod
    def endpoints():
        return list(MWS_ENDPOINTS.values())


def extract_star(star):
    if star:
        tmp = str.replace(star, ',', '.')
        tmp = re.findall(r'(\d+[,.]*\d*)', tmp)
        tmp = float(min(tmp))
        return tmp
    return None


def extract_review_count(review):
    tmp = re.findall('([\d,.]+)', review)
    if len(tmp) > 0:
        tmp = tmp[0]
        tmp = str.replace(tmp, ',', '')
        tmp = str.replace(tmp, '.', '')
        tmp = int(tmp)
        return tmp
    return None


def extract_price(price, thousands_symbol=',', decimal_symbol='.', precision=2):
    tmp = re.findall('([\d,.]+)', price)
    if len(tmp) > 0:
        tmp = tmp[0]
        tmp = str.replace(tmp, thousands_symbol, '')
        tmp = str.replace(tmp, decimal_symbol, '.')
        getcontext().prec = precision
        tmp = Decimal(tmp)
        return tmp
    return None


if __name__ == '__main__':
    e = extract_star('4.6 of 5')
    # e = extract_price('1,199.88')
    print(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : country.py
@Time    : 2021/8/26 14:26
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2021 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import csv
from pathlib import Path


class Country:
    id: str = None
    marketplace_id: str = None
    name: str = None
    continent: str = None
    url: str = None
    url_seller: str = None
    timezone: str = None
    currency: str = None
    mws_endpoint: str = None

    def __init__(self, country: dict):
        self.__dict__.update(country)
        self._dict: dict = country

    @property
    def dict(self):
        return self._dict


class Countries:
    # North America
    BR: Country = None
    CA: Country = None
    MX: Country = None
    US: Country = None
    # Europe region
    AE: Country = None
    DE: Country = None
    EG: Country = None
    ES: Country = None
    FR: Country = None
    GB: Country = None
    IN: Country = None
    IT: Country = None
    NL: Country = None
    PL: Country = None
    SA: Country = None
    SE: Country = None
    TR: Country = None
    # Far East region
    SG: Country = None
    AU: Country = None
    JP: Country = None
    # China
    CN: Country = None
    _ids: list = []
    _dict: dict = {}

    def __init__(self):
        self._load_countries()

    def _load_countries(self):
        folder_path = Path(__file__).parent
        with open(folder_path / 'country.csv', 'r', encoding='utf-8') as f:
            reader = csv.DictReader(f)
            items = list(reader)
        for item in items:
            self.__dict__.update({item['id']: Country(item)})
            self._ids.append(item['id'])
            self._dict.update({item['id']: Country(item).dict})
        # from we_pyutils.t.file import load_yaml_file
        # f = load_yaml_file('country.yaml', multiple=True)

    @property
    def ids(self) -> list:
        return self._ids

    @property
    def marketplace_ids_kv(self) -> dict:
        data = {}
        for id in self._ids:
            data[id] = getattr(self, id).marketplace_id
        return data

    @property
    def marketplace_ids(self) -> list:
        data = []
        for id in self._ids:
            data.append(getattr(self, id).marketplace_id)
        return data

    @property
    def mws_endpoints_kv(self) -> dict:
        data = {}
        for id in self._ids:
            data[id] = getattr(self, id).mws_endpoint
        return data

    @property
    def mws_endpoints(self) -> list:
        data = []
        for id in self._ids:
            data.append(getattr(self, id).mws_endpoint)
        return data

    @property
    def domains(self):
        data = []
        for id in self._ids:
            url = getattr(self, id).url
            if url:
                data.append(url)
        return data

    @property
    def domains_kv(self):
        data = {}
        for id in self._ids:
            if (url := getattr(self, id).url):
                data[id] = url
        return data


if __name__ == '__main__':
    countries = Countries()
    us = countries.US.marketplace_id
    e = countries.mws_endpoints_kv
    e2 = countries.marketplace_ids_kv
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : ip.py
@Time    : 2022/3/14 19:30
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2022 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import json

import requests

from we_pyutils.settings import get_wepyutils_setting


def get_my_ip(url: str = None, tries: int = 3, count: int = 1):
    """
    获取当前IP地址
    :param tries: 尝试次数
    :param count: 当前次数
    :return: IP 或 None
    """
    if not url:
        url = get_wepyutils_setting('get_my_ip_url')
    if count <= tries:
        print(f'Try to get my IP.')
        try:
            headers = {
                'Accept-Encoding': 'application/json',
            }
            req = requests.get(url, headers=headers)
            content = str(req.content, encoding='utf-8')
            my_ip = json.loads(content)['ip']
            if my_ip:
                return my_ip
            print(f'Retrying get ip.')
            return get_my_ip(tries=tries, count=count + 1)
        except Exception as e:
            print(f'Can not get my IP. detail: {e}')
            return get_my_ip(tries=tries, count=count + 1)
    print(f'Get ip: too many attempts')
    return ''


if __name__ == '__main__':
    ip = get_my_ip()
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : settings.py
@Time    : 2022/3/14 19:22
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2022 WingEase Technology Co.,Ltd. All Rights Reserved.
"""

WEPYUTILS_DEFAULTS = {
    'get_my_ip_url': 'https://itool.co/get_ip',
}


def get_wepyutils_setting(name, default=None):
    """Read a setting."""
    # Start with a copy of default settings
    WEPYUTILS = WEPYUTILS_DEFAULTS.copy()

    # Override with user settings from settings.py
    # BOOTSTRAP4.update(getattr(settings, "BOOTSTRAP4", {}))

    # Update use_i18n
    # WEPYUTILS["use_i18n"] = i18n_enabled()

    return WEPYUTILS.get(name, default)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : text.py
@Time    : 2021/8/26 9:26
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2021 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import re


class VariablesReplaceHandler:
    _var_dict: dict = {}
    _content_replaced: str = None

    def __init__(self, content: str, pattern, var_dict: dict, empty_replace: str = ''):
        """
        变量替换处理器
        :param content: 内容文本
        :param pattern: 正则匹配规则
        :param var_dict: 变量字典
        :param empty_replace: 变量不存在替换为
        """
        self._content: str = content
        self._pattern = pattern
        self._var_dict: dict = var_dict if var_dict else {}
        self._empty_replace: str = empty_replace

    def replace(self) -> str:
        self._content_replaced = re.sub(self._pattern, self.replace_methods, self._content)
        return self._content_replaced

    def replace_methods(self, matched) -> str:
        var_name = matched.group('var_name')
        if var_name in self._var_dict:
            return self._var_dict[var_name]
        elif self._empty_replace is not None:
            return self._empty_replace
        else:
            return f'{{{{ {var_name} }}}}'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : __init__.py.py
@Time    : 2021/3/28 16:33
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2021 WingEase Technology Co.,Ltd. All Rights Reserved.
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : env.py
@Time    : 2020-01-26 17:30
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import os


class GetEnv:
    DIRECT = 'Direct'
    DOCKER = 'Docker'
    ENV_PRIORITY = ['Project', 'Docker', 'Home']
    ENV_NAME = {
        'Project': 'project',
        'Docker': 'docker',
        'Home': 'home',
    }
    PROJECT_DIR = None
    HOME_DIR = None
    DOCKER_VOLUME_DIR = None
    DIR_LIST = []
    ENV_FILE_NAME = '.env'

    def __init__(self, dirs=None):
        if dirs and isinstance(dirs, list):
            self.DIR_LIST = dirs

    def get_running_env(self):
        docker_volume_exists = os.path.exists(self.DOCKER_VOLUME_DIR)
        if docker_volume_exists:
            return self.DOCKER
        else:
            return self.DIRECT

    def get_env(self, get_list=None):
        env_list = []
        for dir in self.DIR_LIST:
            ext = self.is_env_file_exists(dir)
            env_list.append(ext)
        if get_list:
            return env_list
        for env_path in env_list:
            if env_path:
                return env_path
        return None

    def is_env_file_exists(self, dir):
        env_path = os.path.join(dir, self.ENV_FILE_NAME)
        return env_path if self.is_file_exists(env_path) else None

    def is_file_exists(self, path):
        return True if os.path.exists(path) else False


if __name__ == '__main__':
    """
    测试用例
    """
    # BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    BASE_DIR = os.path.dirname(os.path.abspath(__file__))
    HOME_DIR = os.path.expanduser('~')
    dir_list = [
        BASE_DIR,
        HOME_DIR,
    ]
    e = GetEnv(dir_list)
    env_path = e.get_env(True)
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : ty_types.py
@Time    : 2021-03-22 9:55
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2021 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import datetime
import random


class ProxyIP:
    """
    自定义太阳代理IP格式
    """

    def __init__(self, ip: str, port=None, expire_time=None, city=None, isp=None, used_times=0):
        self.ip = ip
        self.port = port
        self.expire_time = expire_time
        self.used_times = used_times
        self.city = city
        self.isp = isp

    def __str__(self):
        return f'IP:{self.ip}:{self.port}, Used:{self.used_times} times.'

    def get_ip(self):
        self.used_times_add()
        return self.proxy_str()

    def proxy_str(self):
        return f'{self.ip}:{self.port}'

    def get_used_times(self):
        return self.used_times

    def used_times_add(self, times=1):
        self.used_times += times


class ProxyIPPool:
    def __init__(self, ip_dict: dict = None):
        self.ip_pool = {} if ip_dict is None else ip_dict

    def __len__(self):
        return len(self.ip_pool)

    def enlarge_ip_pool(self, ip_dict: dict):
        self.ip_pool.update(ip_dict)
        return self.ip_pool

    def add_ip(self, ip: ProxyIP):
        self.ip_pool[ip.ip] = ip
        return ip

    def update_ip(self, ip: ProxyIP):
        self.ip_pool[ip.ip] = ip
        return ip

    def remove_ip(self, ip: str):
        del self.ip_pool[ip]

    def random_choice_ip(self, count=1) -> ProxyIP:
        p = list(self.ip_pool.values())
        return random.choice(p)
        # return random.sample(p, count) # 选择多个IP（返回list）

    def get_ip_pool(self):
        return self.ip_pool


class ProxyIPDict:
    def __init__(self, ip_list: list):
        self.ip_dict = {}
        ip_list_initialized = self.init_ip_list(ip_list)
        self.ip_dict.update(ip_list_initialized)

    @staticmethod
    def init_ip_list(ip_list: list) -> dict:
        ip_dict = {}
        if isinstance(ip_list, list):
            for ip_item in ip_list:
                ip = ProxyIP(ip_item['ip'])
                if ip_item['port']:
                    ip.port = ip_item['port']
                if ip_item['expire_time']:
                    ip.expire_time = datetime.datetime.strptime(ip_item['expire_time'], '%Y-%m-%d %H:%M:%S')
                if ip_item['city']:
                    ip.city = ip_item['city']
                if ip_item['isp']:
                    ip.isp = ip_item['isp']
                ip_dict[ip_item['ip']] = ip
        else:
            raise ValueError
        return ip_dict

    def get_ip_dict(self):
        return self.ip_dict


if __name__ == '__main__':
    il = [
        {
            "city": "吉林省四平市",
            "expire_time": "2021-03-22 16:38:04",
            "ip": "221.9.134.198",
            "isp": "联通",
            "port": "4353"
        },
        {
            "city": "吉林省四平市",
            "expire_time": "2021-03-22 16:38:04",
            "ip": "221.9.134.199",
            "isp": "联通",
            "port": "4353"
        }
    ]
    ip_1 = ProxyIP('127.0.0.1', 4353)
    t1 = ip_1.get_ip()
    ip_2 = ProxyIP('127.0.0.2', 2222)
    ip_3 = ProxyIP('127.0.0.3', 3333)
    id = {
        '127.0.0.2': ip_2,
        '127.0.0.3': ip_3,
    }
    ip_dict = ProxyIPDict(il)
    ip_pool = ProxyIPPool()
    ip_pool.add_ip(ip_1)
    ip_pool.enlarge_ip_pool(id)
    l = len(ip_pool)
    # ip_pool.remove_ip('127.0.0.2')
    c1 = ip_pool.random_choice_ip()
    c2 = ip_pool.random_choice_ip(2)
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : oss.py
@Time    : 2020-02-09 8:23
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import os

import oss2

DEFAULT_ENDPOINT = os.getenv('ALIYUN_OSS_ENDPOINT', '')
DEFAULT_ACCESS_KEY_ID = os.getenv('ALIYUN_OSS_ACCESS_KEY_ID', '')
DEFAULT_ACCESS_KEY_SECRET = os.getenv('ALIYUN_OSS_ACCESS_KEY_SECRET', '')
DEFAULT_BUCKET_NAME = os.getenv('ALIYUN_OSS_BUCKET_NAME', '')


class Singleton(object):
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls)
        return cls._instance


class AliyunOssClient(Singleton):
    buckets = {}
    bucket_name = None

    def __init__(self, endpoint=None, key_id=None, key_secret=None):
        self.endpoint = endpoint if endpoint else DEFAULT_ENDPOINT
        self.key_id = key_id if key_id else DEFAULT_ACCESS_KEY_ID
        self.key_secret = key_secret if key_secret else DEFAULT_ACCESS_KEY_SECRET

    def get_oss_bucket(self, bucket_name=None):
        if bucket_name:
            self.bucket_name = bucket_name
        elif self.bucket_name:
            pass
        else:
            self.bucket_name = DEFAULT_BUCKET_NAME
        if bucket_name not in self.buckets:
            auth = oss2.Auth(self.key_id, self.key_secret)
            self.buckets[bucket_name] = oss2.Bucket(auth, self.endpoint, self.bucket_name)
        return self.buckets[bucket_name]


if __name__ == '__main__':
    import environ
    from we_pyutils.env import GetEnv

    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    PROJECT_DIR = os.path.dirname(os.path.dirname(BASE_DIR))
    HOME_DIR = os.path.expanduser('~')
    dir_list = [
        BASE_DIR,
        PROJECT_DIR,
        HOME_DIR,
    ]

    e = GetEnv(dir_list)
    env_path = e.get_env()
    env = environ.Env()
    if env_path:
        env.read_env(env_path)
    end_point = env.str('ALIYUN_OSS_ENDPOINT', '')
    key = env.str('ALIYUN_OSS_ACCESS_KEY_ID', '')
    secret = env.str('ALIYUN_OSS_ACCESS_KEY_SECRET', '')
    bucket_name = env.str('ALIYUN_OSS_BUCKET_NAME', '')
    client = AliyunOssClient(end_point, key, secret)
    bucket = client.get_oss_bucket(bucket_name)
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : ip.py
@Time    : 2020-02-09 8:59
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import json
import os

import requests

DEFAULT_GET_MY_IP_URL = os.getenv('DEFAULT_GET_MY_IP_URL', 'https://www.zenkr.com/tools/getip')


def get_my_ip(url=DEFAULT_GET_MY_IP_URL, tries=3, count=1):
    """
    获取当前IP地址
    :param tries: 尝试次数
    :param count: 当前次数
    :return: IP 或 None
    """
    print(f'Try to get my IP.')
    if count <= tries:
        try:
            req = requests.get(url)
            content = str(req.content, encoding='utf-8')
            my_ip = json.loads(content)['ip']
            return my_ip
        except Exception as e:
            print(f'Can not get my IP. detail: {e}')
            return get_my_ip(tries=tries, count=count + 1)
    return None


if __name__ == '__main__':
    import environ
    from we_pyutils.env import GetEnv

    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    PROJECT_DIR = os.path.dirname(BASE_DIR)
    HOME_DIR = os.path.expanduser('~')
    dir_list = [
        BASE_DIR,
        PROJECT_DIR,
        HOME_DIR,
    ]

    e = GetEnv(dir_list)
    env_path = e.get_env()
    env = environ.Env()
    if env_path:
        env.read_env(env_path)
    url = env.str('GET_MY_IP_URL', '')
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : setup.py
@Time    : 2020-01-26 17:23
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="we_pyutils",
    version="0.2.13",
    author="ZENKR",
    author_email="zenkr@qq.com",
    keywords=("pip", "pyutils"),
    description="WingEase Python Utils",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="GPLv3",
    url="https://github.com/WingEase/we_pyutils",  # 项目相关文件地址，一般是github
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.8",
    # include_package_data=True,
    package_data={
        "we_pyutils": [
            "amazonmws/country.csv",
        ],
    },
    # platforms="any",
    install_requires=[  # 这个项目需要的第三方库
        "requests",
        "requests-toolbelt",
        "Django",
        "django-environ",
        "oss2",
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : astasin.py
@Time    : 2020-02-09 7:03
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""

import datetime
import json
import os

import requests
from requests_toolbelt import MultipartEncoder

DEFAULT_CLIENT_ID = os.getenv('ASTASIN_CLIENT_ID', '')
DEFAULT_CLIENT_SECRET = os.getenv('ASTASIN_CLIENT_SECRET', '')
DEFAULT_USERNAME = os.getenv('ASTASIN_USERNAME', '')
DEFAULT_PASSWORD = os.getenv('ASTASIN_PASSWORD', '')
DEFAULT_AUTH_ENDPOINT = os.getenv('ASTASIN_AUTH_ENDPOINT', 'https://weap.wingease.com')
DEFAULT_ENDPOINT = os.getenv('ASTASIN_ENDPOINT', 'https://astasin.wingease.com')


class Singleton(object):
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls)
        return cls._instance


class AstClient(Singleton):
    authUrl = f'{DEFAULT_AUTH_ENDPOINT}/o/token/'

    accessToken = None
    expiresIn = None
    tokenType = None
    scope = None
    refreshToken = None
    expiredAt = None

    def __init__(self, client_id=None, client_secret=None, username=None, password=None, auth_endpoint=None):
        self._client_id = client_id if client_id else DEFAULT_CLIENT_ID
        self._client_secret = client_secret if client_secret else DEFAULT_CLIENT_SECRET
        self._username = username if username else DEFAULT_USERNAME
        self._password = password if password else DEFAULT_PASSWORD
        self.auth_endpoint = auth_endpoint if auth_endpoint else DEFAULT_AUTH_ENDPOINT
        self.authUrl = f'{self.auth_endpoint}/o/token/'

    def get_token(self):
        now = datetime.datetime.now()
        if self.is_token_expired(now):
            g = self._get_token()
            if 'access_token' in g:
                self.accessToken = g['access_token']
                self.expiresIn = int(g['expires_in'])
                self.tokenType = g['token_type']
                self.scope = g['scope']
                self.refreshToken = g['refresh_token']
                self.expiredAt = now + datetime.timedelta(seconds=self.expiresIn)
            else:
                self.accessToken = None
        return self.accessToken

    def is_token_expired(self, time=None, before=30):
        if self.expiredAt is None:
            return True
        time = datetime.datetime.now() if time is None else time
        delta = (self.expiredAt - time).seconds
        if delta < before:
            return True
        return False

    def _get_token(self):
        body = {
            'grant_type': 'password',
            'client_id': self._client_id,
            'client_secret': self._client_secret,
            'username': self._username,
            'password': self._password,
        }
        m = MultipartEncoder(
            fields=body,
        )
        headers = {
            'Accept': 'application/json',
            'Content-Type': m.content_type
        }
        try:
            r = requests.post(self.authUrl, headers=headers, data=m)
            return json.loads(r.text)
        except Exception as e:
            print(e)
            return None


class AstAsin:
    url = f'{DEFAULT_ENDPOINT}/seller-product-create/'

    def __init__(self, ast_client=None, endpoint=None):
        self.endpoint = endpoint if endpoint else DEFAULT_ENDPOINT
        self.ast_client = ast_client

    def create_seller_product(self, body=None):
        url = f'{self.endpoint}/seller-product-create/'
        headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.ast_client.get_token()}'
        }
        body = body if body else []
        try:
            r = requests.post(url, headers=headers, json=body)
            return json.loads(r.text)
        except Exception as e:
            print(e)
            return False

    def seller_country_subscription_task_get(self, results=10):
        url = f'{self.endpoint}/seller-country-subscription-task/'
        headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.ast_client.get_token()}'
        }
        params = {
            'results': results
        }
        try:
            r = requests.get(url, headers=headers, params=params)
            return json.loads(r.text)
        except Exception as e:
            print(e)
            return False

    def seller_country_subscription_task_post(self, seller_country_subscription_id):
        url = f'{self.endpoint}/seller-country-subscription-task/'
        headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.ast_client.get_token()}'
        }
        params = {
            'id': seller_country_subscription_id
        }
        try:
            r = requests.post(url, headers=headers, params=params)
            return json.loads(r.text)
        except Exception as e:
            print(e)
            return False


if __name__ == '__main__':
    import environ
    from we_pyutils.env import GetEnv

    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    PROJECT_DIR = os.path.dirname(BASE_DIR)
    HOME_DIR = os.path.expanduser('~')
    dir_list = [
        BASE_DIR,
        PROJECT_DIR,
        HOME_DIR,
    ]

    e = GetEnv(dir_list)
    env_path = e.get_env()
    env = environ.Env()
    if env_path:
        env.read_env(env_path)
    client_id = env.str('ASTASIN_CLIENT_ID')
    client_secret = env.str('ASTASIN_CLIENT_SECRET')
    username = env.str('ASTASIN_USERNAME')
    password = env.str('ASTASIN_PASSWORD')
    auth_endpoint = env.str('ASTASIN_AUTH_ENDPOINT')
    endpoint = env.str('ASTASIN_ENDPOINT')
    client = AstClient(client_id=client_id, client_secret=client_secret, username=username,
                       password=password, auth_endpoint=auth_endpoint)
    ast_asin = AstAsin(client, endpoint)
    body = []
    body.append({
        "sellerId": 'seller_id',
        "sellerName": None,
        "asin": 'asin',
        "httpCode": 200,
        "countryCode": 'DE',
        'memo': 'TEST',
    })
    resp = ast_asin.create_seller_product(body)
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : ty.py
@Time    : 2020-02-09 8:56
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""
import datetime
import json
import os
import time
from urllib.request import urlopen

from we_pyutils.t.ip import get_my_ip
from .ty_types import ProxyIP

DEFAULT_GET_IP_URL = os.getenv('PROXY_TY_GET_IP_URL', '')
DEFAULT_GET_PACK_INFO_URL = os.getenv('PROXY_TY_GET_PACK_INFO_URL', '')
DEFAULT_ADD_WHITE_IP_URL = os.getenv('PROXY_TY_ADD_WHITE_IP_URL', '')
DEFAULT_DEL_WHITE_IP_URL = os.getenv('PROXY_TY_DEL_WHITE_IP_URL', '')
DEFAULT_ACCOUNT_NAME = os.getenv('PROXY_TY_ACCOUNT_NAME', '')
DEFAULT_ACCOUNT_KEY = os.getenv('PROXY_TY_ACCOUNT_KEY', '')
DEFAULT_NEEK = os.getenv('PROXY_TY_NEEK', '')
DEFAULT_APPKEY = os.getenv('PROXY_TY_APPKEY', '')


class Singleton(object):
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls)
        return cls._instance


class ProxyTYClient(Singleton):
    get_pack_info_count = 0
    max_ip_used_times = 65
    packs_info = {}

    def __init__(self, get_ip_url=None, get_pack_info_url=None,
                 add_white_ip_url=None, del_white_ip_url=None,
                 account_name=None, account_key=None, neek=None, appkey=None
                 ):
        self.get_ip_url = get_ip_url if get_ip_url else DEFAULT_GET_IP_URL
        self.get_pack_info_url = get_pack_info_url if get_pack_info_url else DEFAULT_GET_PACK_INFO_URL
        self.add_white_ip_url = add_white_ip_url if add_white_ip_url else DEFAULT_ADD_WHITE_IP_URL
        self.del_white_ip_url = del_white_ip_url if del_white_ip_url else DEFAULT_DEL_WHITE_IP_URL
        self.account_name = account_name if account_name else DEFAULT_ACCOUNT_NAME
        self._account_key = account_key if account_key else DEFAULT_ACCOUNT_KEY
        self.neek = neek if neek else DEFAULT_NEEK
        self._appkey = appkey if appkey else DEFAULT_APPKEY

    def get_ips(self, pack_id, ip_nums=1, proxy_regions=None, times=0):
        if times >= 10:
            return False
        if proxy_regions is None:
            proxy_regions = []
        print(f'Get new proxy IPs ... ({times} times)')
        try:
            url = f'{self.get_ip_url}' + \
                  f'?num={ip_nums}&type=2&pro=0&city=0&yys=0&port=11&' + \
                  f'pack={pack_id}&ts=1&ys=1&cs=1&lb=1&sb=0&pb=4&mr=0&regions=' + \
                  ','.join(proxy_regions)
            res = urlopen(url, timeout=2).read().decode('utf-8')
            res_json = json.loads(res)

            if res_json.get('code', -1) > 0:
                my_ip = get_my_ip()
                if my_ip:
                    self.add_white_ip(my_ip)
                time.sleep(5)
                return self.get_ips(pack_id, ip_nums, proxy_regions, times=times + 1)
            elif res_json.get('success', False):
                ips = res_json.get('data', {})
                return ips
            return False
        except Exception as e:
            print(f'Get new proxy IPs FAILED. {e}')
            return False

    def get_pack_info(self, count=0):
        try:
            if len(self.packs_info) > 0:
                return self.packs_info
            if count >= 10:
                return False
            self.get_pack_info_count += 1
            print(f'Get Proxy Pack Info ({count} times)')
            gpi_url = f'{self.get_pack_info_url}?neek={self.neek}&appkey={self._appkey}'
            gpi_json = urlopen(gpi_url, timeout=4).read().decode('utf-8')
            self.packs_info = json.loads(gpi_json)
            return self.packs_info
        except Exception as e:
            print(f'No Available Packs!!! 10s Later Retry...({count} times)')
            print(e)
            my_ip = get_my_ip()
            if my_ip:
                self.add_white_ip(my_ip)
            time.sleep(10)
            return self.get_pack_info(count=count + 1)

    def check_and_add_white_ip(self):
        my_ip = get_my_ip()
        if my_ip:
            self.add_white_ip(my_ip)
        time.sleep(5)
        return my_ip

    def add_white_ip(self, ip):
        try:
            add_ip_url = f'{self.add_white_ip_url}?neek={self.account_name}&appkey={self._account_key}&white={ip}'
            res = urlopen(add_ip_url, timeout=2).read().decode('utf-8')
            res_json = json.loads(res)
            if res_json['success']:
                print(f'Add white ip:{ip}')
                return True
        except Exception as e:
            print(f'Add white ip ({ip}) FAILED.')
            print(e)
            return False

    def del_white_ip(self):
        pass

    @staticmethod
    def check_pack_usable(pack):
        all_num = int(pack['all_num'])
        balance = int(pack['balance'])
        name = pack['name']
        near_fill_time = pack['near_fill_time']
        now_timestamps = time.time()
        if name == '包年套餐':
            if all_num > 0:
                return True
            return False
        elif name == '单月包量套餐':
            if (balance > 0) and (near_fill_time is None or now_timestamps < near_fill_time):
                return True
        else:
            return False

    def check_ip_usable(self, ip: ProxyIP, expire_seconds: int = 120):
        if ip.used_times >= self.max_ip_used_times:
            return False
        now = datetime.datetime.now()
        if ip.expire_time <= now:
            return False
        c = ip.expire_time - now
        if c.seconds < expire_seconds:
            return False
        return True


if __name__ == '__main__':
    import environ
    from we_pyutils.env import GetEnv

    BASE_DIR = os.path.dirname(os.path.abspath(__file__))
    PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.dirname(BASE_DIR)))
    HOME_DIR = os.path.expanduser('~')
    dir_list = [
        BASE_DIR,
        PROJECT_DIR,
        HOME_DIR,
    ]

    e = GetEnv(dir_list)
    env_path = e.get_env()
    env = environ.Env()
    if env_path:
        env.read_env(env_path)
    get_ip_url = env.str('PROXY_TY_GET_IP_URL', '')
    get_pack_info_url = env.str('PROXY_TY_GET_PACK_INFO_URL', '')
    add_white_ip_url = env.str('PROXY_TY_ADD_WHITE_IP_URL', '')
    del_white_ip_url = env.str('PROXY_TY_DEL_WHITE_IP_URL', '')
    account_name = env.str('PROXY_TY_ACCOUNT_NAME', '')
    account_key = env.str('PROXY_TY_ACCOUNT_KEY', '')
    neek = env.str('PROXY_TY_NEEK', '')
    appkey = env.str('PROXY_TY_APPKEY', '')

    proxy = ProxyTYClient(
        get_ip_url, get_pack_info_url,
        add_white_ip_url, del_white_ip_url,
        account_name, account_key, neek, appkey
    )
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : file.py
@Time    : 2020-02-09 7:01
@Author  : ZENKR
@Email   : zenkr@qq.com
@Software: PyCharm
@Desc    :
@license : Copyright (c) 2020 WingEase Technology Co.,Ltd. All Rights Reserved.
"""

from os import listdir
from os.path import isfile, join, splitext
from typing import Union

from ruamel import yaml


# 获取文件列表（支持指定扩展名）
def get_file_list(dir_path, extension_list=None):
    file_list = []
    for f in listdir(dir_path):
        if isfile(join(dir_path, f)):
            if extension_list is None or extension_list == []:
                file_list.append(f)
            else:
                if splitext(f)[1] in extension_list:
                    file_list.append(f)
    return file_list


def load_yaml_file(file: str, multiple: bool = False) -> Union[dict, list]:
    """
    读取YAML文件
    @param file: 文件路径
    @param multiple: 同文件内是否包含多个文档
    @return: 字典或列表
    """
    with open(file, 'r', encoding="utf-8") as f:
        yaml_data = f.read()
    if multiple:
        yaml_load = yaml.load_all(yaml_data, Loader=yaml.Loader)
    else:
        yaml_load = yaml.load(yaml_data, Loader=yaml.Loader)
    data_list = []
    for item in yaml_load:
        data_list.append(item)
    return data_list


def export_yaml_file(content: Union[dict, list], path: str, multiple: bool = False):
    """
    保存YAML文件
    @param content: 文件内容
    @param path: 保存文件路径
    @param multiple: 同文件内是否包含多个文档
    @return:
    """
    with open(path, "w", encoding="utf-8") as f:
        if multiple:
            yaml.dump_all(content, f, Dumper=yaml.RoundTripDumper)
        else:
            yaml.dump(content, f, Dumper=yaml.RoundTripDumper)


if __name__ == '__main__':
    f = load_yaml_file('./data/demo.yaml', multiple=True)
    export_yaml_file(f, './data/demo_export.yaml', multiple=True)
    print(0)
